import 'package:flutter/material.dart';
import 'dart:math' as math;

// ==============
// Models
// ==============

math.Random random = new math.Random();

class Tile {
  Color color;
  String index;

  Tile(this.index,this.color);
}

// ==============
// Widgets
// ==============

class TileWidget extends StatelessWidget {
  final Tile tile;

  TileWidget(this.tile);

  @override
  Widget build(BuildContext context) {
    return this.coloredBox();
  }

  Widget coloredBox() {
    return Container(
        color: tile.color,
        child: Align(
            child: Text(tile.index.toString(),),
            alignment: Alignment.center,
      )
    );
  }
}

int emptyIndex=0;

List<Widget>createTileWidgets(double slide){
  List<Widget> tiles = <Widget>[];
  int max=slide.toInt()*slide.toInt();
  tiles.add(TileWidget(Tile("Vide",Colors.white)));
  for(int i=0;i<max-1;i++){
    tiles.add(TileWidget(Tile((i+1).toString(), Colors.grey)));
  }
  return tiles;
}


void main() => runApp(new MaterialApp(home: PositionedTiles()));

class PositionedTiles extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => PositionedTilesState();
}

class PositionedTilesState extends State<PositionedTiles> {
  TextStyle textStyle = TextStyle(fontSize: 16, fontWeight: FontWeight.bold,);
  var appBar= AppBar(title: Text('Exo 6'),);
  double _currentSlideValue=4;
  List<Widget> tiles=createTileWidgets(4);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: appBar,
      body: ListView(children: [
        Container(
          height: ((MediaQuery.of(context).size.height)*0.7),
          child:GridView.builder(
            padding: const EdgeInsets.all(10),
            itemCount: _currentSlideValue.toInt()*_currentSlideValue.toInt(),
            gridDelegate:SliverGridDelegateWithFixedCrossAxisCount(
              crossAxisCount: _currentSlideValue.toInt(),
              crossAxisSpacing: 5.0,
              mainAxisSpacing: 5.0,
            ),
            itemBuilder: (BuildContext context, int index) {
              return InkWell(
                child: tiles[index],
                onTap: (){
                  swapTiles(index,emptyIndex);
                },
              );
            },
          ),
        ),
        Row(children: <Widget>[
              Container(width: 90,child:Text('Taille:', style: textStyle,textAlign: TextAlign.center,),padding: EdgeInsets.fromLTRB(8, 0, 0, 0),),
              Container(height:((MediaQuery.of(context).size.height)/7)-appBar.preferredSize.height,width: MediaQuery.of(context).size.width-90,child: Slider(
                  value: _currentSlideValue,
                  min: 2, max: 7, divisions: 5,
                  label: _currentSlideValue.round().toString(),
                  onChanged: (double value) {
                    setState(() {
                      _currentSlideValue = value;
                      tiles=createTileWidgets(_currentSlideValue);
                      //initialSize=_currentSlideValue;
                    });
                  }
               ),
              ),
            ]
        ),
      ],
      )
    );
  }

  swapTiles(int current, int empty) {
    setState(() {
      //print(current);
      if(current==empty+1 && current%_currentSlideValue!=0){
        emptyIndex=current;
        tiles.insert(empty, tiles.removeAt(current));
      }else if(current==empty-1  && empty%_currentSlideValue!=0){
        emptyIndex=current;
        tiles.insert(empty, tiles.removeAt(current));
      }
      else if(current==empty+_currentSlideValue){
        emptyIndex=current;
        tiles.insert(current, tiles.removeAt(empty));
        tiles.insert(empty, tiles.removeAt(current-1));
      }else if(current==empty-_currentSlideValue){
        emptyIndex=current;
        tiles.insert(current, tiles.removeAt(empty));
        tiles.insert(empty, tiles.removeAt(current+1));
      }else print("Can't move this");
    });
  }

}